
//
// List contents of any 'INI' file.
// May be [bootdrive]:OS2/OS2.INI, [bootdrive]:OS2SYS.INI or 'program.ini'
//
// Author: Rob Hamerling
//
// Maximum 1 commandline parameter expected:
//     S             list only System (OS2SYS.INI)
//     U             list only User info (OS2.INI)
//  <program.ini>    list contents of 'program.ini'
//
//  (default: list both OS2.INI and OS2SYS.INI)
//
// Notes: 1. Some return code checking omitted for brevity!
//        2. Entries starting with 'PM_' are listed, by uncommenting
//           two indicated lines these may be suppressed.
//        3. Contains code to remove all entries of some specific programs
//           (keys and profiles of 'Norman' and 'PMcomm')
//           The lines are commented out to avoid undesired removal!


#define  INCL_WINSHELLDATA
#define  INCL_WINERRORS
#include <os2.h>

#include <ctype.h>
#include <stdio.h>
#include <string.h>


int  main(int argc, char *argv[]) {

  HINI    hini = HINI_PROFILE;          // sys + user profile
  HAB     hab;                          // window hook anchor block
  char   *pPgmNames = NULL, *pPgm;
  char   *pKeyNames = NULL, *pKey;
  char   *pKeyDataBuf = NULL;
  ULONG   ulPgmNamesBufSize;
  ULONG   ulKeyNamesBufSize;
  ULONG   ulKeyDataBufSize = 4096;
  ULONG   ulPgmNameLen;
  ULONG   ulKeyNameLen;
  ULONG   ulKeyLen;
  ULONG   i;
  BOOL    rc;


  printf("\n Profile Data of ");
  if (argc > 1) {
    if (argv[1][1] == '\0') {
      if (argv[1][0] == 'u' || argv[1][0] == 'U') {
        hini = HINI_USERPROFILE;                // user profile only
        printf("OS2.INI");
        }
      else if (argv[1][0] == 's' || argv[1][0] == 'S') {
        hini = HINI_SYSTEMPROFILE;              // system profile only
        printf("OS2SYS.INI");
        }
      }
    else {
      printf(argv[1]);                          // private profile
      hab = WinInitialize(0);
      hini = PrfOpenProfile(hab, argv[1]);
      if (hini == NULLHANDLE) {
        printf("\nError opening private profile file: %s)\n", argv[1]);
        return 1;
        }
      }
    }
  else
    printf("OS2SYS.INI and OS2.INI");           // both
  printf("\n\n");

  PrfQueryProfileSize(hini, NULL, NULL, &ulPgmNamesBufSize);
  DosAllocMem((PPVOID)&pPgmNames, ulPgmNamesBufSize,
               PAG_COMMIT | PAG_READ | PAG_WRITE);
  DosAllocMem((PPVOID)&pKeyDataBuf, ulKeyDataBufSize,
               PAG_COMMIT | PAG_READ | PAG_WRITE);

  rc = PrfQueryProfileData(hini, NULL, NULL,
                           (void *)pPgmNames, &ulPgmNamesBufSize);
  if (rc == FALSE) {
    printf(" PrfQueryProfileData error, rc 0x%08lX\n",
             WinGetLastError(hab));
    return 1;
    }

  pPgm = pPgmNames;
  while ((ulPgmNameLen = strlen(pPgm)) > 0) {

// Uncomment following line if you want to suppress 'PM_' entries
//  if (strncmp(pPgm, "PM_", 3)) {      // skip system related profiles

      printf("\n%s\n", pPgm);
      PrfQueryProfileSize(hini, pPgm, NULL, &ulKeyNamesBufSize);

      if (ulKeyNamesBufSize > 0) {
        DosAllocMem((PPVOID)&pKeyNames, ulKeyNamesBufSize,
                     PAG_COMMIT | PAG_READ | PAG_WRITE);
        rc = PrfQueryProfileData(hini, pPgm, NULL,
                                 (void *)pKeyNames, &ulKeyNamesBufSize);
        pKey = pKeyNames;

        while ((ulKeyNameLen = strlen(pKey)) > 0) {
          PrfQueryProfileSize(hini, pPgm, pKey, &ulKeyLen);
          printf("  * %s\n", pKey);
          if (ulKeyLen > ulKeyDataBufSize) {
            DosFreeMem(pKeyDataBuf);
            ulKeyDataBufSize = ulKeyLen;
            DosAllocMem((PPVOID)&pKeyDataBuf, ulKeyDataBufSize,
                         PAG_COMMIT | PAG_READ | PAG_WRITE);
            }
          ulKeyLen = ulKeyDataBufSize;
          PrfQueryProfileData(hini, pPgm, pKey, pKeyDataBuf, &ulKeyLen);
          printf("    ");
          for (i=0; i<ulKeyLen; ++i)
            printf("%02X", pKeyDataBuf[i]);
          printf("\n    ");
          for (i=0; i<ulKeyLen; ++i)
            printf(" %c", (pKeyDataBuf[i]>=32 && pKeyDataBuf[i]<=127) ?
                                    pKeyDataBuf[i] : '.' );
          printf("\n");

// Code to remove ALL keys of specific programs from OS2.INI and OS2SYS.INI

//        if ( (strnicmp(pPgm, "Norman", 6) == 0 ||
//              strnicmp(pPgm, "PMComm", 6) == 0)   &&
//             (hini == HINI_USERPROFILE   ||
//              hini == HINI_SYSTEMPROFILE ||
//              hini == HINI_PROFILE)) {
//          if (PrfWriteProfileData(hini, pPgm, pKey, NULL, 0) == TRUE)
//            printf("  *** key removed ***\n");
//          else
//            printf("  *** failed to remove key, rc 0x%08lX ***\n",
//                   WinGetLastError(hab));
//          }

// --- end of key-remove code -------------------------------------------

          pKey += ulKeyNameLen + 1;
          }
        DosFreeMem(pKeyNames);

// Code to remove profile of specific programs from OS2.INI and OS2SYS.INI

//      if ( (strnicmp(pPgm, "Norman", 6) == 0 ||
//            strnicmp(pPgm, "PMComm", 6) == 0)    &&
//          (hini == HINI_USERPROFILE   ||
//           hini == HINI_SYSTEMPROFILE ||
//           hini == HINI_PROFILE)) {
//        if (PrfWriteProfileData(hini, pPgm, NULL, NULL, 0) == TRUE)
//          printf("  *** profile removed ***\n");
//        else {
//          printf("  *** failed to remove profile, rc 0x%08lX ***\n",
//                 WinGetLastError(hab));
//          }
//        }

// --- end of program-remove code ---------------------------------------


        }

// Uncomment following line if you want to suppress 'PM_' entries
//    }

    pPgm += ulPgmNameLen + 1;                   // to next program

    }

  printf("\n\n");

  DosFreeMem(pPgmNames);

  return 0;

  }


